@@ -112,7 +112,6 @@ struct _ClockData {
 
 	GtkListStore *cities_store;
         GtkWidget *cities_section;
-        GtkWidget *map_section;
         GtkWidget *map_widget;
 
 	/* preferences */
@@ -729,7 +728,7 @@ create_map_section (ClockData *cd)
         ClockMap *map;
 
         if (cd->map_widget) {
-                gtk_widget_destroy (GTK_WIDGET (cd->map_section));
+                gtk_widget_destroy (cd->map_widget);
                 cd->map_widget = NULL;
         }
 
@@ -737,16 +736,15 @@ create_map_section (ClockData *cd)
         g_signal_connect (map, "need-locations",
                           G_CALLBACK (map_need_locations_cb), cd);
 
-        cd->map_section = gtk_alignment_new (0, 0, 1, 1);
         cd->map_widget = GTK_WIDGET (map);
 
-        gtk_container_add (GTK_CONTAINER (cd->map_section), cd->map_widget);
+        gtk_widget_set_margin_top (cd->map_widget, 1);
+        gtk_widget_set_margin_bottom (cd->map_widget, 1);
+        gtk_widget_set_margin_start (cd->map_widget, 1);
+        gtk_widget_set_margin_end (cd->map_widget, 1);
 
-        gtk_alignment_set_padding (GTK_ALIGNMENT (cd->map_section), 1, 1, 1, 1);
-
-        gtk_box_pack_start (GTK_BOX (cd->clock_vbox), cd->map_section, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (cd->clock_vbox), cd->map_widget, TRUE, TRUE, 0);
         gtk_widget_show (cd->map_widget);
-        gtk_widget_show (cd->map_section);
 }
 
 static void
@@ -757,7 +755,6 @@ update_calendar_popup (ClockData *cd)
                         gtk_widget_destroy (cd->calendar_popup);
                         cd->calendar_popup = NULL;
                         cd->cities_section = NULL;
-                        cd->map_section = NULL;
                         cd->map_widget = NULL;
 			cd->clock_vbox = NULL;
 			
@@ -1216,7 +1213,6 @@ location_set_current_cb (ClockLocation *loc,
 	if (cd->map_widget)
 		clock_map_refresh (CLOCK_MAP (cd->map_widget));
         update_location_tiles (cd);
-	save_cities_store (cd);	
 }
 
 static void
@@ -1333,9 +1329,6 @@ fill_clock_applet (PanelApplet *applet)
 
 	create_clock_widget (cd);
 
-#ifndef CLOCK_INPROCESS
-	gtk_window_set_default_icon_name (CLOCK_ICON);
-#endif
 	gtk_widget_show (cd->applet);
 
 	/* FIXME: Update this comment. */
@@ -1352,9 +1345,6 @@ fill_clock_applet (PanelApplet *applet)
 			  G_CALLBACK (panel_button_change_pixel_size),
 			  cd);
 
-	panel_applet_set_background_widget (PANEL_APPLET (cd->applet),
-					    GTK_WIDGET (cd->applet));
-
         action_group = g_simple_action_group_new ();
 	g_action_map_add_action_entries (G_ACTION_MAP (action_group),
 	                                 clock_menu_actions,
@@ -1623,8 +1613,7 @@ edit_clear (ClockData *cd)
         GtkWidget *lon_combo = _clock_get_widget (cd, "edit-location-longitude-combo");
 
         /* clear out the old data */
-        /* TODO: reinstate after bumping to gweather >= 3.10.1 */
-        //gweather_location_entry_set_location (cd->location_entry, NULL);
+        gweather_location_entry_set_location (cd->location_entry, NULL);
         gweather_timezone_menu_set_tzid (cd->zone_combo, NULL);
 
         gtk_entry_set_text (GTK_ENTRY (lat_entry), "");
@@ -2102,14 +2091,7 @@ clock_factory (PanelApplet *applet,
 	return retval;
 }
 
-#ifdef CLOCK_INPROCESS
 PANEL_APPLET_IN_PROCESS_FACTORY ("ClockAppletFactory",
                                  PANEL_TYPE_APPLET,
                                  clock_factory,
                                  NULL)
-#else
-PANEL_APPLET_OUT_PROCESS_FACTORY ("ClockAppletFactory",
-                                  PANEL_TYPE_APPLET,
-                                  clock_factory,
-                                  NULL)
-#endif
