@@ -52,8 +52,9 @@ typedef struct {
         GSettings *settings;
 } TasklistData;
 
-static void display_properties_dialog (GtkAction    *action,
-				       TasklistData *tasklist);
+static void display_properties_dialog (GSimpleAction *action,
+                                       GVariant      *parameter,
+                                       gpointer       user_data);
 
 static void
 tasklist_update (TasklistData *tasklist)
@@ -113,6 +114,8 @@ applet_change_orient (PanelApplet       *applet,
   
 	tasklist->orientation = new_orient;
 
+	wnck_tasklist_set_orientation (WNCK_TASKLIST (tasklist->tasklist), new_orient);
+
 	tasklist_update (tasklist);
 }
 
@@ -126,19 +129,6 @@ applet_change_background (PanelApplet     *applet,
                                                          : GTK_RELIEF_NORMAL);
 }
 
-static void
-applet_change_pixel_size (PanelApplet  *applet,
-			  gint          size,
-			  TasklistData *tasklist)
-{
-	if (tasklist->size == size)
-		return;
-
-	tasklist->size = size;
-	
-	tasklist_update (tasklist);
-}
-
 static void
 destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
 {
@@ -151,10 +141,8 @@ destroy_tasklist(GtkWidget * widget, TasklistData *tasklist)
         g_free (tasklist);
 }
 
-static const GtkActionEntry tasklist_menu_actions [] = {
-	{ "TasklistPreferences", GTK_STOCK_PROPERTIES, N_("_Preferences"),
-	  NULL, NULL,
-	  G_CALLBACK (display_properties_dialog) }
+static const GActionEntry tasklist_menu_actions [] = {
+        { "preferences", display_properties_dialog, NULL, NULL, NULL },
 };
 
 static void
@@ -267,7 +255,7 @@ move_unminimized_windows_changed (GSettings    *settings,
 }
 
 static void
-setup_gconf (TasklistData *tasklist)
+setup_gsettings (TasklistData *tasklist)
 {
         tasklist->settings =
           panel_applet_settings_new (PANEL_APPLET (tasklist->applet),
@@ -288,14 +276,28 @@ applet_size_allocate (GtkWidget      *widget,
                       GtkAllocation  *allocation,
                       TasklistData   *tasklist)
 {
-	int len;
+	gint len, size;
 	const int *size_hints;
+	PanelAppletOrient orient = panel_applet_get_orient (PANEL_APPLET (tasklist->applet));
 
 	size_hints = wnck_tasklist_get_size_hint_list (WNCK_TASKLIST (tasklist->tasklist), &len);
 	g_assert (len % 2 == 0);
 
         panel_applet_set_size_hints (PANEL_APPLET (tasklist->applet),
 		                     size_hints, len, 0);
+
+	if (orient == PANEL_APPLET_ORIENT_UP || orient == PANEL_APPLET_ORIENT_DOWN) {
+		size = allocation->height;
+	} else {
+		size = allocation->width;
+	}
+
+	if (tasklist->size == size)
+		return;
+
+	tasklist->size = size;
+
+	tasklist_update (tasklist);
 }
 
 static GdkPixbuf*
@@ -352,8 +354,8 @@ gboolean
 window_list_applet_fill (PanelApplet *applet)
 {
 	TasklistData *tasklist;
-	GtkActionGroup *action_group;
-	GtkAction *action;
+	GSimpleActionGroup *action_group;
+	GAction *action;
 
 	tasklist = g_new0 (TasklistData, 1);
 
@@ -364,15 +366,12 @@ window_list_applet_fill (PanelApplet *applet)
 				PANEL_APPLET_EXPAND_MINOR |
 				PANEL_APPLET_HAS_HANDLE);
 
-	panel_applet_add_preferences (applet, "/schemas/apps/window_list_applet/prefs", NULL);
-
-	setup_gconf (tasklist);
+	setup_gsettings (tasklist);
 
 	tasklist->include_all_workspaces = g_settings_get_boolean (tasklist->settings, "display-all-workspaces");
 	tasklist->grouping = g_settings_get_enum (tasklist->settings, "group-windows");
 	tasklist->move_unminimized_windows = g_settings_get_boolean (tasklist->settings, "move-unminimized-windows");
 
-	tasklist->size = panel_applet_get_size (applet);
 	switch (panel_applet_get_orient (applet)) {
 	case PANEL_APPLET_ORIENT_LEFT:
 	case PANEL_APPLET_ORIENT_RIGHT:
@@ -387,11 +386,10 @@ window_list_applet_fill (PanelApplet *applet)
 
 	tasklist->tasklist = wnck_tasklist_new ();
 
-        wnck_tasklist_set_icon_loader (WNCK_TASKLIST (tasklist->tasklist),
-                                       icon_loader_func,
-                                       tasklist,
-                                       NULL);
-        
+	wnck_tasklist_set_orientation (WNCK_TASKLIST (tasklist->tasklist), tasklist->orientation);
+	wnck_tasklist_set_icon_loader (WNCK_TASKLIST (tasklist->tasklist),
+	                               icon_loader_func, tasklist, NULL);
+
 	g_signal_connect (G_OBJECT (tasklist->tasklist), "destroy",
 			  G_CALLBACK (destroy_tasklist),
 			  tasklist);
@@ -411,31 +409,27 @@ window_list_applet_fill (PanelApplet *applet)
 			  "change_orient",
 			  G_CALLBACK (applet_change_orient),
 			  tasklist);
-	g_signal_connect (G_OBJECT (tasklist->applet),
-			  "change_size",
-			  G_CALLBACK (applet_change_pixel_size),
-			  tasklist);
 	g_signal_connect (G_OBJECT (tasklist->applet),
 			  "change_background",
 			  G_CALLBACK (applet_change_background),
 			  tasklist);
 
-	panel_applet_set_background_widget (PANEL_APPLET (tasklist->applet),
-					    GTK_WIDGET (tasklist->applet));
-
-	action_group = gtk_action_group_new ("Tasklist Applet Actions");
-	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (action_group,
-				      tasklist_menu_actions,
-				      G_N_ELEMENTS (tasklist_menu_actions),
-				      tasklist);
+	action_group = g_simple_action_group_new ();
+	g_action_map_add_action_entries (G_ACTION_MAP (action_group),
+	                                 tasklist_menu_actions,
+	                                 G_N_ELEMENTS (tasklist_menu_actions),
+	                                 tasklist);
 	panel_applet_setup_menu_from_resource (PANEL_APPLET (tasklist->applet),
 					       WNCKLET_RESOURCE_PATH "window-list-menu.xml",
-					       action_group);
+					       action_group,
+					       GETTEXT_PACKAGE);
 
-	action = gtk_action_group_get_action (action_group, "TasklistPreferences");
+	gtk_widget_insert_action_group (GTK_WIDGET (applet), "tasklist",
+	                                G_ACTION_GROUP (action_group));
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (action_group), "preferences");
 	g_object_bind_property (tasklist->applet, "locked-down",
-				action, "visible",
+				action, "enabled",
 				G_BINDING_DEFAULT|G_BINDING_INVERT_BOOLEAN|G_BINDING_SYNC_CREATE);
 
 	g_object_unref (action_group);
@@ -578,9 +572,12 @@ setup_dialog (GtkBuilder   *builder,
 
 
 static void 
-display_properties_dialog (GtkAction    *action,
-			   TasklistData *tasklist)
+display_properties_dialog (GSimpleAction *action,
+                           GVariant      *parameter,
+                           gpointer       user_data)
 {
+	TasklistData *tasklist = (TasklistData *) user_data;
+
 	if (tasklist->properties_dialog == NULL) {
 		GtkBuilder *builder;
 
