@@ -45,6 +45,8 @@ struct _PanelAppletsManagerDBusPrivate
 };
 
 typedef gint (* ActivateAppletFunc) (void);
+typedef GtkWidget * (* GetAppletWidgetFunc) (const gchar *factory_id,
+                                             guint        uid);
 
 typedef struct _PanelAppletFactoryInfo {
 	gchar              *id;
@@ -52,6 +54,7 @@ typedef struct _PanelAppletFactoryInfo {
 	gboolean            in_process;
 	GModule            *module;
 	ActivateAppletFunc  activate_applet;
+	GetAppletWidgetFunc get_applet_widget;
 	guint               n_applets;
 
 	gchar              *srcdir;
@@ -394,6 +397,7 @@ panel_applets_manager_dbus_factory_activate (PanelAppletsManager *manager,
 {
 	PanelAppletFactoryInfo *info;
 	ActivateAppletFunc      activate_applet;
+	GetAppletWidgetFunc     get_applet_widget;
 
 	info = get_applet_factory_info (manager, iid);
 	if (!info)
@@ -433,6 +437,15 @@ panel_applets_manager_dbus_factory_activate (PanelAppletsManager *manager,
 		return FALSE;
 	}
 
+	if (!g_module_symbol (info->module, "panel_applet_get_applet_widget", (gpointer *) &get_applet_widget)) {
+		/* FIXME: use a GError? */
+		g_warning ("Failed to load applet %s: %s", iid, g_module_error ());
+		g_module_close (info->module);
+		info->module = NULL;
+
+		return FALSE;
+	}
+
 	/* Activate the applet */
 	if (activate_applet () != 0) {
 		/* FIXME: use a GError? */
@@ -443,6 +456,7 @@ panel_applets_manager_dbus_factory_activate (PanelAppletsManager *manager,
 		return FALSE;
 	}
 	info->activate_applet = activate_applet;
+	info->get_applet_widget = get_applet_widget;
 
 	info->n_applets = 1;
 
@@ -549,6 +563,20 @@ panel_applets_manager_dbus_load_applet (PanelAppletsManager         *manager,
 	return panel_applet_frame_dbus_load (iid, frame_act);
 }
 
+static GtkWidget *
+panel_applets_manager_dbus_get_applet_widget (PanelAppletsManager *manager,
+                                              const gchar         *iid,
+                                              guint                uid)
+{
+	PanelAppletFactoryInfo *info;
+
+	info = get_applet_factory_info (manager, iid);
+	if (!info)
+		return NULL;
+
+	return info->get_applet_widget (info->id, uid);
+}
+
 static void
 panel_applets_manager_dbus_finalize (GObject *object)
 {
@@ -597,6 +625,7 @@ panel_applets_manager_dbus_class_init (PanelAppletsManagerDBusClass *class)
 	manager_class->get_applet_info = panel_applets_manager_dbus_get_applet_info;
 	manager_class->get_applet_info_from_old_id = panel_applets_manager_dbus_get_applet_info_from_old_id;
 	manager_class->load_applet = panel_applets_manager_dbus_load_applet;
+	manager_class->get_applet_widget = panel_applets_manager_dbus_get_applet_widget;
 
 	g_type_class_add_private (class, sizeof (PanelAppletsManagerDBusPrivate));
 }
