@@ -34,11 +34,11 @@
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-launch.h>
 #include <libpanel-util/panel-screensaver.h>
+#include <libpanel-util/panel-end-session-dialog.h>
 #include <libpanel-util/panel-session-manager.h>
 #include <libpanel-util/panel-show.h>
 
 #include "applet.h"
-#include "panel-gconf.h"
 #include "panel-typebuiltins.h"
 #include "panel-force-quit.h"
 #include "panel-util.h"
@@ -67,7 +67,12 @@ struct _PanelActionButtonPrivate {
 	guint                  dnd_enabled : 1;
 };
 
-static GConfEnumStringPair panel_action_type_map [] = {
+typedef struct {
+	gint enum_value;
+	const gchar* str;
+} PanelEnumStringPair;
+
+static PanelEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_NONE,           "none"           },
 	{ PANEL_ACTION_LOCK,           "lock"           },
 	{ PANEL_ACTION_LOGOUT,         "logout"         },
@@ -75,10 +80,43 @@ static GConfEnumStringPair panel_action_type_map [] = {
 	{ PANEL_ACTION_SEARCH,         "search"         },
 	{ PANEL_ACTION_FORCE_QUIT,     "force-quit"     },
 	{ PANEL_ACTION_CONNECT_SERVER, "connect-server" },
+	{ PANEL_ACTION_HIBERNATE,      "hibernate"      },
+	{ PANEL_ACTION_SUSPEND,        "suspend"        },
+	{ PANEL_ACTION_HYBRID_SLEEP,   "hybrid-sleep"   },
+	{ PANEL_ACTION_REBOOT,         "reboot"         },
 	{ PANEL_ACTION_SHUTDOWN,       "shutdown"       },
 	{ 0,                           NULL             },
 };
 
+static gboolean
+panel_string_to_enum (const gchar *str, gint *enum_value_retloc)
+{
+	int i = 0;
+	while (panel_action_type_map[i].str != NULL)
+	{
+		if (g_ascii_strcasecmp (panel_action_type_map[i].str, str) == 0)
+		{
+			*enum_value_retloc = panel_action_type_map[i].enum_value;
+			return TRUE;
+		}
+		++i;
+	}
+	return FALSE;
+}
+
+static const gchar*
+panel_enum_to_string (gint enum_value)
+{
+	int i = 0;
+	while (panel_action_type_map[i].str != NULL)
+	{
+		if (panel_action_type_map[i].enum_value == enum_value)
+			return panel_action_type_map[i].str;
+		++i;
+	}
+	return NULL;
+}
+
 /* Lock Screen
  */
 static void
@@ -121,19 +159,16 @@ panel_action_lock_setup_menu (PanelActionButton *button)
 {
 	panel_applet_add_callback (button->priv->info,
 				   "lock",
-				   NULL,
 				   _("_Lock Screen"),
 				   panel_action_lock_is_enabled);
 
 	panel_applet_add_callback (button->priv->info,
 				   "activate",
-				   NULL,
 				   _("_Activate Screensaver"),
 				   NULL);
 
 	panel_applet_add_callback (button->priv->info,
 				   "prefs",
-				   GTK_STOCK_PROPERTIES,
 				   _("_Properties"),
 				   screensaver_properties_enabled);
 }
@@ -169,6 +204,81 @@ panel_action_logout (GtkWidget *widget)
 					      PANEL_SESSION_MANAGER_LOGOUT_MODE_NORMAL);
 }
 
+static void
+panel_action_hibernate (GtkWidget *widget)
+{
+	PanelEndSessionDialog *dialog;
+
+	dialog = panel_end_session_dialog_get ();
+	panel_end_session_dialog_request_hibernate (dialog);
+}
+
+static gboolean
+panel_action_hibernate_is_disabled (void)
+{
+	PanelEndSessionDialog *dialog;
+
+	if (panel_lockdown_get_disable_log_out_s ())
+		return TRUE;
+
+	dialog = panel_end_session_dialog_get ();
+
+	return (!panel_end_session_dialog_is_hibernate_available (dialog));
+}
+
+static void
+panel_action_suspend (GtkWidget *widget)
+{
+	PanelEndSessionDialog *dialog;
+
+	dialog = panel_end_session_dialog_get ();
+	panel_end_session_dialog_request_suspend (dialog);
+}
+
+static gboolean
+panel_action_suspend_is_disabled (void)
+{
+	PanelEndSessionDialog *dialog;
+
+	if (panel_lockdown_get_disable_log_out_s ())
+		return TRUE;
+
+	dialog = panel_end_session_dialog_get ();
+
+	return (!panel_end_session_dialog_is_suspend_available (dialog));
+}
+
+static void
+panel_action_hybrid_sleep  (GtkWidget *widget)
+{
+	PanelEndSessionDialog *dialog;
+
+	dialog = panel_end_session_dialog_get ();
+	panel_end_session_dialog_request_hybrid_sleep (dialog);
+}
+
+static gboolean
+panel_action_hybrid_sleep_is_disabled (void)
+{
+	PanelEndSessionDialog *dialog;
+
+	if (panel_lockdown_get_disable_log_out_s ())
+		return TRUE;
+
+	dialog = panel_end_session_dialog_get ();
+
+	return (!panel_end_session_dialog_is_hybrid_sleep_available (dialog));
+}
+
+static void
+panel_action_reboot (GtkWidget *widget)
+{
+	PanelSessionManager *manager;
+
+	manager = panel_session_manager_get ();
+	panel_session_manager_request_reboot (manager);
+}
+
 static void
 panel_action_shutdown (GtkWidget *widget)
 {
@@ -342,6 +452,42 @@ static PanelAction actions [] = {
 		"ACTION:connect-server:NEW",
 		panel_action_connect_server, NULL, NULL, NULL
 	},
+	{
+		PANEL_ACTION_HIBERNATE,
+		NULL,
+		N_("Hibernate"),
+		NULL,
+		"ACTION:hibernate:NEW",
+		panel_action_hibernate, NULL, NULL,
+		panel_action_hibernate_is_disabled
+	},
+	{
+		PANEL_ACTION_SUSPEND,
+		NULL,
+		N_("Suspend"),
+		NULL,
+		"ACTION:suspend:NEW",
+		panel_action_suspend, NULL, NULL,
+		panel_action_suspend_is_disabled
+	},
+	{
+		PANEL_ACTION_HYBRID_SLEEP,
+		NULL,
+		N_("Hybrid sleep"),
+		NULL,
+		"ACTION:hybrid-sleep:NEW",
+		panel_action_hybrid_sleep, NULL, NULL,
+		panel_action_hybrid_sleep_is_disabled
+	},
+	{
+		PANEL_ACTION_REBOOT,
+		NULL,
+		N_("Restart"),
+		N_("Restart the computer"),
+		"ACTION:reboot:NEW",
+		panel_action_reboot, NULL, NULL,
+		panel_action_shutdown_reboot_is_disabled
+	},
 	{
 		PANEL_ACTION_SHUTDOWN,
 		PANEL_ICON_SHUTDOWN,
@@ -506,7 +652,7 @@ panel_action_button_drag_data_get (GtkWidget          *widget,
 	button = PANEL_ACTION_BUTTON (widget);
 
 	drag_data = g_strdup_printf ("ACTION:%s:%d", 
-				     gconf_enum_to_string (panel_action_type_map, button->priv->type),
+				     panel_enum_to_string (button->priv->type),
 				     panel_find_applet_index (widget));
 
 	gtk_selection_data_set (
@@ -654,12 +800,8 @@ panel_action_button_create (PanelToplevel         *toplevel,
 			    int                    pack_index,
 			    PanelActionButtonType  type)
 {
-	const char *detail;
-
-	detail = gconf_enum_to_string (panel_action_type_map, type);
-
 	panel_layout_object_create (PANEL_OBJECT_ACTION,
-				    detail,
+				    panel_enum_to_string (type),
 				    panel_toplevel_get_id (toplevel),
 				    pack_type, pack_index);
 }
@@ -725,7 +867,7 @@ panel_action_button_load_from_drag (PanelToplevel       *toplevel,
 		return retval;
 	}
 
-	if (!gconf_string_to_enum (panel_action_type_map, elements [1], (gpointer) &type)) {
+	if (!panel_string_to_enum (elements [1], (gpointer) &type)) {
 		g_strfreev (elements);
 		return retval;
 	}
