@@ -47,12 +47,12 @@
 #include <libpanel-util/panel-gtk.h>
 #include <libpanel-util/panel-keyfile.h>
 #include <libpanel-util/panel-launch.h>
+#include <libpanel-util/panel-end-session-dialog.h>
 #include <libpanel-util/panel-session-manager.h>
 #include <libpanel-util/panel-show.h>
 
 #include "menu.h"
 #include "panel-action-button.h"
-#include "panel-gconf.h"
 #include "panel-globals.h"
 #include "panel-icon-names.h"
 #include "panel-lockdown.h"
@@ -60,17 +60,13 @@
 #include "panel-recent.h"
 #include "panel-stock-icons.h"
 #include "panel-util.h"
+#include "panel-schemas.h"
 
-#define DESKTOP_IS_HOME_DIR_DIR "/apps/nautilus/preferences"
-#define DESKTOP_IS_HOME_DIR_KEY "/apps/nautilus/preferences/desktop_is_home_dir"
-#define NAMES_DIR               "/apps/nautilus/desktop"
-#define HOME_NAME_KEY           "/apps/nautilus/desktop/home_icon_name"
-#define COMPUTER_NAME_KEY       "/apps/nautilus/desktop/computer_icon_name"
 #define MAX_ITEMS_OR_SUBMENU    8
 #define MAX_BOOKMARK_ITEMS      100
 
-G_DEFINE_TYPE (PanelPlaceMenuItem, panel_place_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
-G_DEFINE_TYPE (PanelDesktopMenuItem, panel_desktop_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
+G_DEFINE_TYPE (PanelPlaceMenuItem, panel_place_menu_item, PANEL_TYPE_IMAGE_MENU_ITEM)
+G_DEFINE_TYPE (PanelDesktopMenuItem, panel_desktop_menu_item, PANEL_TYPE_IMAGE_MENU_ITEM)
 
 #define PANEL_PLACE_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_PLACE_MENU_ITEM, PanelPlaceMenuItemPrivate))
 #define PANEL_DESKTOP_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_DESKTOP_MENU_ITEM, PanelDesktopMenuItemPrivate))
@@ -329,13 +325,13 @@ panel_menu_item_desktop_new (char      *path,
 		name = g_strdup (force_name);
 
 	if (use_icon) {
-		item = panel_image_menu_item_new ();
+		item = panel_image_menu_item_new2 ();
         } else {
-		item = gtk_image_menu_item_new ();
+		item = panel_image_menu_item_new ();
 	}
 
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-				   icon, NULL, NULL, name);
+				   icon, NULL, name);
 
 	panel_util_set_tooltip_text (item, comment);
 
@@ -379,10 +375,10 @@ panel_menu_item_uri_new (const char *uri,
 	GtkWidget *item;
 	char      *user_data;
 
-	item = panel_image_menu_item_new ();
+	item = panel_image_menu_item_new2 ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   icon_name, NULL, gicon,
+				   icon_name, gicon,
 				   title);
 
 	panel_util_set_tooltip_text (item, tooltip);
@@ -410,11 +406,11 @@ panel_menu_items_create_action_item_full (PanelActionButtonType  action_type,
 	if (!create_even_if_disabled && panel_action_get_is_disabled (action_type))
 		return NULL;
 
-	item = gtk_image_menu_item_new ();
+	item = panel_image_menu_item_new ();
         setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
 				   panel_action_get_icon_name (action_type),
-				   NULL, NULL,
+				   NULL,
 				   label ? label : panel_action_get_text (action_type));
 
 	panel_util_set_tooltip_text (item,
@@ -445,32 +441,63 @@ static void
 panel_menu_item_activate_switch_user (GtkWidget *menuitem,
 				      gpointer   user_data)
 {
-	GdkScreen *screen;
-	GAppInfo  *app_info;
-
 	if (panel_lockdown_get_disable_switch_user_s ())
 		return;
 
-	screen = gtk_widget_get_screen (GTK_WIDGET (menuitem));
-	app_info = g_app_info_create_from_commandline (GDM_FLEXISERVER_COMMAND " " GDM_FLEXISERVER_ARGS,
-						       GDM_FLEXISERVER_COMMAND,
-						       G_APP_INFO_CREATE_NONE,
-						       NULL);
+	/* If running under LightDM switch to the greeter using dbus */
+	if (g_getenv("XDG_SEAT_PATH")) {
+		GDBusConnection *bus;
+		GError  *error = NULL;
+		GVariant *result = NULL;
 
-	if (app_info) {
-		GdkAppLaunchContext *launch_context;
-		GdkDisplay          *display;
+		bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &error);
+		if (error)
+			g_warning ("Failed to get system bus: %s", error->message);
+		g_clear_error (&error);
 
-		display = gdk_screen_get_display (screen);
-		launch_context = gdk_display_get_app_launch_context (display);
-		gdk_app_launch_context_set_screen (launch_context, screen);
+		if (bus)
+			result = g_dbus_connection_call_sync (bus,
+							      "org.freedesktop.DisplayManager",
+							      g_getenv ("XDG_SEAT_PATH"),
+							      "org.freedesktop.DisplayManager.Seat",
+							      "SwitchToGreeter",
+							      g_variant_new ("()"),
+							      G_VARIANT_TYPE ("()"),
+							      G_DBUS_CALL_FLAGS_NONE,
+							      -1,
+							      NULL,
+							      &error);
+                if (error)
+                        g_warning ("Failed to switch to greeter: %s", error->message);
+                g_clear_error (&error);
 
-		g_app_info_launch (app_info, NULL,
-				   G_APP_LAUNCH_CONTEXT (launch_context),
-				   NULL);
+                if (result)
+                        g_variant_unref (result);
+        } else {
+		GdkScreen *screen;
+		GAppInfo  *app_info;
 
-		g_object_unref (launch_context);
-		g_object_unref (app_info);
+		screen = gtk_widget_get_screen (GTK_WIDGET (menuitem));
+		app_info = g_app_info_create_from_commandline (GDM_FLEXISERVER_COMMAND " " GDM_FLEXISERVER_ARGS,
+							       GDM_FLEXISERVER_COMMAND,
+							       G_APP_INFO_CREATE_NONE,
+							       NULL);
+
+		if (app_info) {
+			GdkAppLaunchContext *launch_context;
+			GdkDisplay          *display;
+
+			display = gdk_screen_get_display (screen);
+			launch_context = gdk_display_get_app_launch_context (display);
+			gdk_app_launch_context_set_screen (launch_context, screen);
+
+			g_app_info_launch (app_info, NULL,
+					   G_APP_LAUNCH_CONTEXT (launch_context),
+					   NULL);
+
+			g_object_unref (launch_context);
+			g_object_unref (app_info);
+		}
 	}
 }
 
@@ -480,13 +507,13 @@ panel_menu_items_create_switch_user (gboolean use_icon)
 	GtkWidget *item;
 
 	if (use_icon) {
-		item = panel_image_menu_item_new ();
+		item = panel_image_menu_item_new2 ();
         } else {
-		item = gtk_image_menu_item_new ();
+		item = panel_image_menu_item_new ();
 	}
 
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-				   NULL, NULL, NULL, _("Switch User"));
+				   NULL, NULL, _("Switch User"));
 
 	g_signal_connect (item, "activate",
 			  G_CALLBACK (panel_menu_item_activate_switch_user),
@@ -613,9 +640,9 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 	} else {
 		GtkWidget *item;
 
-		item = gtk_image_menu_item_new ();
+		item = panel_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-					   PANEL_ICON_BOOKMARKS, NULL, NULL,
+					   PANEL_ICON_BOOKMARKS, NULL,
 					   _("Bookmarks"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -746,10 +773,10 @@ panel_menu_item_drive_new (GDrive *drive)
 	icon = g_drive_get_icon (drive);
 	title = g_drive_get_name (drive);
 
-	item = panel_image_menu_item_new ();
+	item = panel_image_menu_item_new2 ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   NULL, NULL, icon,
+				   NULL, icon,
 				   title);
 	g_object_unref (icon);
 
@@ -845,10 +872,10 @@ panel_menu_item_volume_new (GVolume *volume)
 	icon = g_volume_get_icon (volume);
 	title = g_volume_get_name (volume);
 
-	item = panel_image_menu_item_new ();
+	item = panel_image_menu_item_new2 ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
-				   NULL, NULL, icon,
+				   NULL, icon,
 				   title);
 	g_object_unref (icon);
 
@@ -1056,10 +1083,10 @@ panel_place_menu_item_append_local_gio (PanelPlaceMenuItem *place_item,
 	} else {
 		GtkWidget  *item;
 
-		item = gtk_image_menu_item_new ();
+		item = panel_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 					   PANEL_ICON_REMOVABLE_MEDIA,
-					   NULL, NULL,
+					   NULL,
 					   _("Removable Media"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1147,10 +1174,10 @@ panel_place_menu_item_append_remote_gio (PanelPlaceMenuItem *place_item,
 	} else {
 		GtkWidget  *item;
 
-		item = panel_image_menu_item_new ();
+		item = panel_image_menu_item_new2 ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 					   PANEL_ICON_NETWORK_SERVER,
-					   NULL, NULL,
+					   NULL,
 					   _("Network Places"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
@@ -1181,7 +1208,6 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 {
 	GtkWidget *places_menu;
 	GtkWidget *item;
-	char      *gconf_name;
 	char      *name;
 	char      *uri;
 	GFile     *file;
@@ -1202,45 +1228,32 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	g_free (name);
 	g_free (uri);
 
-	if (!gconf_client_get_bool (panel_gconf_get_client (),
-				    DESKTOP_IS_HOME_DIR_KEY,
-				    NULL)) {
-		file = g_file_new_for_path (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
-		uri = g_file_get_uri (file);
-		g_object_unref (file);
-		
-		item = panel_menu_item_uri_new (
-				/* FIXME: if the dir changes, we'd need to update the drag data since the uri is not the same */
-				uri, PANEL_ICON_DESKTOP, NULL,
-				/* Translators: Desktop is used here as in
-				 * "Desktop Folder" (this is not the Desktop
-				 * environment). */
-				C_("Desktop Folder", "Desktop"),
-				_("Open the contents of your desktop in a folder"),
-				G_CALLBACK (activate_desktop_uri));
-		gtk_menu_shell_append (GTK_MENU_SHELL (places_menu), item);
+	file = g_file_new_for_path (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
 
-		g_free (uri);
-	}
+	item = panel_menu_item_uri_new (
+			/* FIXME: if the dir changes, we'd need to update the drag data since the uri is not the same */
+			uri, PANEL_ICON_DESKTOP, NULL,
+			/* Translators: Desktop is used here as in
+			 * "Desktop Folder" (this is not the Desktop
+			 * environment). */
+			C_("Desktop Folder", "Desktop"),
+			_("Open the contents of your desktop in a folder"),
+			G_CALLBACK (activate_desktop_uri));
+	gtk_menu_shell_append (GTK_MENU_SHELL (places_menu), item);
+
+	g_free (uri);
 
 	panel_place_menu_item_append_gtk_bookmarks (places_menu);
 	add_menu_separator (places_menu);
 
-	gconf_name = gconf_client_get_string (panel_gconf_get_client (),
-					      COMPUTER_NAME_KEY,
-					      NULL);
-
-	if (gconf_name == NULL) {
-		gconf_name = g_strdup (_("Computer"));
-	}
-
 	item = panel_menu_item_uri_new ("computer://",
 					PANEL_ICON_COMPUTER, NULL,
-					gconf_name,
+					_("Computer"),
 					_("Browse all local and remote disks and folders accessible from this computer"),
 					G_CALLBACK (activate_uri));
 	gtk_menu_shell_append (GTK_MENU_SHELL (places_menu), item);
-	g_free (gconf_name);
 
 	panel_place_menu_item_append_local_gio (place_item, places_menu);
 	add_menu_separator (places_menu);
@@ -1294,10 +1307,7 @@ panel_place_menu_item_recreate_menu (GtkWidget *widget)
 }
 
 static void
-panel_place_menu_item_key_changed (GConfClient *client,
-				   guint        cnxn_id,
-				   GConfEntry  *entry,
-				   GtkWidget   *place_item)
+panel_place_menu_item_key_changed (GSettings *settings, const gchar *key, GtkWidget *place_item)
 {
 	panel_place_menu_item_recreate_menu (place_item);
 }
@@ -1393,13 +1403,6 @@ panel_place_menu_item_finalize (GObject *object)
 {
 	PanelPlaceMenuItem *menuitem = (PanelPlaceMenuItem *) object;
 
-	gconf_client_remove_dir (panel_gconf_get_client (),
-				 DESKTOP_IS_HOME_DIR_DIR,
-				 NULL);
-	gconf_client_remove_dir (panel_gconf_get_client (),
-				 NAMES_DIR,
-				 NULL);
-
 	if (menuitem->priv->bookmarks_monitor != NULL) {
 		g_file_monitor_cancel (menuitem->priv->bookmarks_monitor);
 		g_object_unref (menuitem->priv->bookmarks_monitor);
@@ -1486,27 +1489,13 @@ panel_place_menu_item_init (PanelPlaceMenuItem *menuitem)
 	GFile *bookmark;
 	char  *bookmarks_filename;
 	GError *error;
+	GSettings *settings;
 
 	menuitem->priv = PANEL_PLACE_MENU_ITEM_GET_PRIVATE (menuitem);
 
-	gconf_client_add_dir (panel_gconf_get_client (),
-			      DESKTOP_IS_HOME_DIR_DIR,
-			      GCONF_CLIENT_PRELOAD_NONE,
-			      NULL);
-	gconf_client_add_dir (panel_gconf_get_client (),
-			      NAMES_DIR,
-			      GCONF_CLIENT_PRELOAD_NONE,
-			      NULL);
-
-	panel_gconf_notify_add_while_alive (HOME_NAME_KEY,
-					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
-					    G_OBJECT (menuitem));
-	panel_gconf_notify_add_while_alive (DESKTOP_IS_HOME_DIR_KEY,
-					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
-					    G_OBJECT (menuitem));
-	panel_gconf_notify_add_while_alive (COMPUTER_NAME_KEY,
-					    (GConfClientNotifyFunc) panel_place_menu_item_key_changed,
-					    G_OBJECT (menuitem));
+	settings = g_settings_new (GNOME_NAUTILUS_DESKTOP_SCHEMA);
+	g_signal_connect (settings, "changed::" GNOME_NAUTILUS_DESKTOP_HOME_ICON_NAME_KEY,
+	                  G_CALLBACK (panel_place_menu_item_key_changed), menuitem);
 
 	menuitem->priv->recent_manager = gtk_recent_manager_get_default ();
 
@@ -1622,17 +1611,21 @@ panel_place_menu_item_new (gboolean use_image,
 	if (in_menubar) {
 		gtk_menu_item_set_label (GTK_MENU_ITEM (menuitem), name);
 		if (use_image) {
+			GtkIconSize icon_size;
 			GtkWidget *image;
-			image = gtk_image_new_from_icon_name (icon_name,
-							      panel_menu_bar_object_icon_get_size ());
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
-						       image);
+
+			icon_size = panel_menu_bar_object_icon_get_size ();
+			image = gtk_image_new_from_icon_name (icon_name, icon_size);
+
+			gtk_image_set_pixel_size (GTK_IMAGE (image),
+			                          panel_menu_bar_object_icon_get_pixel_size (icon_size));
+			panel_image_menu_item_set_image (PANEL_IMAGE_MENU_ITEM (menuitem), image);
 		}
 	} else {
 		if (use_image)
 			setup_menu_item_with_icon (GTK_WIDGET (menuitem),
 						   panel_menu_icon_get_size (),
-						   icon_name, NULL, NULL,
+						   icon_name, NULL,
 						   name);
 		else
 			setup_menuitem (GTK_WIDGET (menuitem),
@@ -1673,7 +1666,7 @@ panel_desktop_menu_item_on_presence_changed (PanelSessionManager             *ma
 		break;
 	}
 
-	image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (desktop_item));
+	image = panel_image_menu_item_get_image (PANEL_IMAGE_MENU_ITEM (desktop_item));
 	/* we only have an image if we are specifically using an icon for this
 	 * menu */
 	if (image) {
@@ -1730,8 +1723,9 @@ panel_desktop_menu_item_new (gboolean use_image,
 			GtkWidget *image;
 			image = gtk_image_new_from_icon_name (icon_name,
 							      menuitem->priv->icon_size);
-			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
-						       image);
+			gtk_image_set_pixel_size (GTK_IMAGE (image),
+			                          panel_menu_bar_object_icon_get_pixel_size (menuitem->priv->icon_size));
+			panel_image_menu_item_set_image (PANEL_IMAGE_MENU_ITEM (menuitem), image);
 		}
 	} else {
 		menuitem->priv->icon_size = panel_menu_icon_get_size ();
@@ -1739,7 +1733,7 @@ panel_desktop_menu_item_new (gboolean use_image,
 		if (use_image)
 			setup_menu_item_with_icon (GTK_WIDGET (menuitem),
 						   menuitem->priv->icon_size,
-						   icon_name, NULL, NULL,
+						   icon_name, NULL,
 						   name);
 		else
 			setup_menuitem (GTK_WIDGET (menuitem),
@@ -1749,8 +1743,7 @@ panel_desktop_menu_item_new (gboolean use_image,
 
 #ifdef HAVE_TELEPATHY_GLIB
 	if (use_image)
-		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menuitem),
-							   TRUE);
+		panel_image_menu_item_set_always_show_image (PANEL_IMAGE_MENU_ITEM (menuitem), TRUE);
 #endif
 
 	g_free (name);
@@ -1823,6 +1816,8 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 	GList      *children;
 	GList      *last;
 	GtkWidget  *item;
+	GtkWidget *separator;
+	gboolean   hide_separator;
 
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
 	last = g_list_last (children);
@@ -1875,29 +1870,88 @@ panel_menu_items_append_lock_logout (GtkWidget *menu)
 					G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
 	}
 
-	/* FIXME: should be dynamic */
-	if (panel_session_manager_is_shutdown_available (panel_session_manager_get ())) {
-		item = panel_menu_items_create_action_item_full (PANEL_ACTION_SHUTDOWN,
+	/* Separator */
+	separator = add_menu_separator (menu);
+	hide_separator = TRUE;
+
+	g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+	                        separator, "visible",
+	                        G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+
+	/* Hibernate */
+	if (panel_end_session_dialog_is_hibernate_available (panel_end_session_dialog_get ())) {
+		item = panel_menu_items_create_action_item_full (PANEL_ACTION_HIBERNATE,
 								 NULL, NULL, TRUE);
 		if (item != NULL) {
-			GtkWidget *sep;
-
-			sep = add_menu_separator (menu);
+			hide_separator = FALSE;
 
 			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
-			g_object_bind_property (panel_lockdown_get (),
-						"disable-log-out",
-						sep,
-						"visible",
-						G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
-			g_object_bind_property (panel_lockdown_get (),
-						"disable-log-out",
-						item,
-						"visible",
-						G_BINDING_SYNC_CREATE|G_BINDING_INVERT_BOOLEAN);
+			g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+			                        item, "visible",
+			                        G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
 		}
 	}
+
+	/* Suspend */
+	if (panel_end_session_dialog_is_suspend_available (panel_end_session_dialog_get ())) {
+		item = panel_menu_items_create_action_item_full (PANEL_ACTION_SUSPEND,
+								 NULL, NULL, TRUE);
+		if (item != NULL) {
+			hide_separator = FALSE;
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+			                        item, "visible",
+			                        G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+		}
+	}
+
+	/* Hubrid Sleep */
+	if (panel_end_session_dialog_is_hybrid_sleep_available (panel_end_session_dialog_get ())) {
+		item = panel_menu_items_create_action_item_full (PANEL_ACTION_HYBRID_SLEEP,
+								 NULL, NULL, TRUE);
+		if (item != NULL) {
+			hide_separator = FALSE;
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+			                        item, "visible",
+			                        G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+		}
+	}
+
+	/* FIXME: should be dynamic */
+	if (panel_session_manager_is_shutdown_available (panel_session_manager_get ())) {
+		item = panel_menu_items_create_action_item_full (PANEL_ACTION_REBOOT,
+	                                                 NULL, NULL, TRUE);
+		if (item != NULL) {
+			hide_separator = FALSE;
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+				                item, "visible",
+				                G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+		}
+
+		item = panel_menu_items_create_action_item_full (PANEL_ACTION_SHUTDOWN,
+								 NULL, NULL, TRUE);
+		if (item != NULL) {
+			hide_separator = FALSE;
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+			g_object_bind_property (panel_lockdown_get (), "disable-log-out",
+				                item, "visible",
+				                G_BINDING_SYNC_CREATE | G_BINDING_INVERT_BOOLEAN);
+		}
+	}
+
+	if (hide_separator)
+		g_object_set (separator, "visible", FALSE, NULL);
 }
 
 void
