@@ -1514,6 +1514,18 @@ panel_applet_set_background_string (PanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "background");
 }
 
+static void
+panel_applet_style_updated (GtkWidget *widget)
+{
+  PanelApplet *applet;
+
+  applet = PANEL_APPLET (widget);
+
+  GTK_WIDGET_CLASS (panel_applet_parent_class)->style_updated (widget);
+
+  panel_applet_handle_background (applet);
+}
+
 static void
 panel_applet_update_background_for_widget (GtkWidget       *widget,
 					   cairo_pattern_t *pattern)
@@ -1539,6 +1551,8 @@ panel_applet_handle_background (PanelApplet *applet)
 	if (applet->priv->background_widget)
 		panel_applet_update_background_for_widget (applet->priv->background_widget,
 							   pattern);
+	else
+		panel_applet_update_background_for_widget (GTK_WIDGET (applet), pattern);
 
         g_signal_emit (G_OBJECT (applet),
                         panel_applet_signals [CHANGE_BACKGROUND],
@@ -1834,6 +1848,7 @@ panel_applet_class_init (PanelAppletClass *klass)
         widget_class->get_preferred_height = panel_applet_get_preferred_height;
 	widget_class->size_allocate = panel_applet_size_allocate;
 	widget_class->draw = panel_applet_draw;
+	widget_class->style_updated = panel_applet_style_updated;
 	widget_class->focus = panel_applet_focus;
 	widget_class->realize = panel_applet_realize;
 
